/**
 * Copyright 2011 J Adam Crain (jadamcrain@gmail.com)
 *
 * Licensed to J Adam Crain under one or more contributor license agreements.
 * See the NOTICE file distributed with this work for additional information
 * regarding copyright ownership. J Adam Crain licenses this file to you under
 * the Apache License, Version 2.0 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of the License
 * at:
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package com.automatak.osgi4s

import com.automatak.executor4s.{ Failure, Result }
import java.nio.file.{ Paths, Path }

case class Config(map: Map[String, String]) {

  def require[A](pid: String, prop: String)(fun: String => A): Result[A] = {
    val fullName = pid + "." + prop
    map.get(fullName) match {
      case Some(x) => Result(fun(x))
      case None => Failure("Required property not found: " + fullName)
    }
  }

  def requireBoolean(pid: String, prop: String): Result[Boolean] = require(pid, prop)(_.toBoolean)
  def requireInt(pid: String, prop: String): Result[Int] = require(pid, prop)(_.toInt)
  def requireString(pid: String, prop: String): Result[String] = require(pid, prop)(s => s)
  def requirePath(pid: String, prop: String): Result[Path] = require(pid, prop)(s => Paths.get(s))

}
