/**
 * Copyright 2011 J Adam Crain (jadamcrain@gmail.com)
 *
 * Licensed to J Adam Crain under one or more contributor license agreements.
 * See the NOTICE file distributed with this work for additional information
 * regarding copyright ownership. J Adam Crain licenses this file to you under
 * the Apache License, Version 2.0 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of the License
 * at:
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package com.automatak.osgi4s.bundles

import org.osgi.framework.{ BundleContext, BundleActivator }
import com.automatak.executor4s.{ Failure, Success, Result }

trait ConfiguredBundleActivator[C] extends BundleActivator {

  protected def readConfig(ctx: BundleContext): Result[C]

  override def start(ctx: BundleContext): Unit = {
    readConfig(ctx) match {
      case Success(config) =>
        start(config, ctx)
      case Failure(ex) => throw ex // let the framework catch this, we don't want the lifecycle to change
    }
  }

  override def stop(ctx: BundleContext): Unit = {
    shutdown()
  }

  protected def start(config: C, ctx: BundleContext): Unit

  // synchronously stop the running bundle, must be idempotent!
  protected def shutdown(): Unit
}
