/**
 * Copyright 2011 J Adam Crain (jadamcrain@gmail.com)
 *
 * Licensed to J Adam Crain under one or more contributor license agreements.
 * See the NOTICE file distributed with this work for additional information
 * regarding copyright ownership. J Adam Crain licenses this file to you under
 * the Apache License, Version 2.0 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of the License
 * at:
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package com.automatak.osgi4s.bundles

import org.osgi.framework.BundleContext
import com.automatak.osgi4s.dsl._
import com.automatak.osgi4s.impl.{ RemovedServiceEvent, ModifiedServiceEvent, AddingServiceEvent }

trait DoubleRequirementBundleActivator[C, I1 <: AnyRef, I2 <: AnyRef] extends SingleRequirementBundleActivator[C, I1] {

  protected var service2: Option[I2] = None

  protected def classForI2: Class[I2]

  override def start(config: C, ctx: BundleContext): Unit = {
    ctx.watchService(classForI2) { event =>
      event match {
        case AddingServiceEvent(service, props) =>
          service2 = Some(service)
          checkForStart(config, ctx)
        case ModifiedServiceEvent(service, props) =>
          shutdown()
          service2 = Some(service)
          checkForStart(config: C, ctx)
        case RemovedServiceEvent(service, props) =>
          shutdown()
          service2 = None
      }
    }
    super.start(config, ctx)
  }

}
