/**
 * Copyright 2011 J Adam Crain (jadamcrain@gmail.com)
 *
 * Licensed to J Adam Crain under one or more contributor license agreements.
 * See the NOTICE file distributed with this work for additional information
 * regarding copyright ownership. J Adam Crain licenses this file to you under
 * the Apache License, Version 2.0 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of the License
 * at:
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package com.automatak.osgi4s.bundles

import org.osgi.framework.BundleContext
import com.automatak.osgi4s.dsl._
import com.automatak.osgi4s.impl.{ RemovedServiceEvent, ModifiedServiceEvent, AddingServiceEvent }

trait SingleRequirementBundleActivator[C, I1 <: AnyRef] extends ConfiguredBundleActivator[C] {

  protected var service1: Option[I1] = None

  protected def classForI1: Class[I1]

  override protected def start(config: C, ctx: BundleContext): Unit = {
    ctx.watchService(classForI1) { event =>
      event match {
        case AddingServiceEvent(service, props) =>
          service1 = Some(service)
          checkForStart(config, ctx)
        case ModifiedServiceEvent(service, props) =>
          shutdown()
          service1 = Some(service)
          checkForStart(config, ctx)
        case RemovedServiceEvent(service, props) =>
          service1 = None
          shutdown()
      }
    }
  }

  protected def checkForStart(config: C, ctx: BundleContext)

}
