/**
 * Copyright 2011 J Adam Crain (jadamcrain@gmail.com)
 *
 * Licensed to J Adam Crain under one or more contributor license agreements.
 * See the NOTICE file distributed with this work for additional information
 * regarding copyright ownership. J Adam Crain licenses this file to you under
 * the Apache License, Version 2.0 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of the License
 * at:
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package com.automatak.osgi4s.impl

import org.osgi.framework.{ ServiceReference, BundleContext }
import org.osgi.util.tracker.ServiceTracker

sealed trait ServiceEvent[I]
final case class AddingServiceEvent[I](service: I, props: Map[String, String]) extends ServiceEvent[I]
final case class ModifiedServiceEvent[I](service: I, props: Map[String, String]) extends ServiceEvent[I]
final case class RemovedServiceEvent[I](service: I, props: Map[String, String]) extends ServiceEvent[I]

private[impl] final class DefaultServiceTracker[I <: AnyRef](ctx: BundleContext, klass: Class[I])(handler: ServiceEvent[I] => Unit)
    extends ServiceTracker(ctx, klass.getName, null) {

  import ServiceReferenceHelpers.propertiesOf

  override def addingService(serviceReference: ServiceReference) = {
    val service = ctx.getService(serviceReference).asInstanceOf[I]
    handler(AddingServiceEvent(service, propertiesOf(serviceReference)))
    service
  }

  override def modifiedService(serviceReference: ServiceReference, service: AnyRef) = {
    handler(ModifiedServiceEvent(service.asInstanceOf[I], propertiesOf(serviceReference)))
  }

  override def removedService(serviceReference: ServiceReference, service: AnyRef) = {
    handler(RemovedServiceEvent(service.asInstanceOf[I], propertiesOf(serviceReference)))
    context.ungetService(serviceReference)
  }
}

